<?php
/**
 * @version $Id: manager.php 1413 2012-02-02 09:09:11Z roosit $
 * @package Abricos
 * @subpackage Company
 * @copyright Copyright (C) 2008 Abricos. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin (roosit@abricos.org)
 */

require_once 'dbquery.php';

class CompanyManager extends Ab_ModuleManager {
	
	/**
	 * 
	 * @var CompanyModule
	 */
	public $module = null;
	
	public function __construct(CompanyModule $module){
		parent::__construct($module);
		
		Abricos::$user->GetManager();
	}
	
	public function IsAdminRole(){
		return $this->IsRoleEnable(CompanyAction::ADMIN);
	}
	
	public function IsViewRole(){
		return $this->IsRoleEnable(CompanyAction::VIEW);
	}
	
	public function IsWriteRole(){
		return $this->IsRoleEnable(CompanyAction::WRITE);
	}
	
	public function IsManagerRole(){
		return $this->IsRoleEnable(CompanyAction::MANAGER);
	}
	
	private function GetCalendarManager(){
		$modCal = Abricos::GetModule('calendar');
		if (empty($modCal)){ return null; } 
		return $modCal->GetManager();
	}
	
	public function DSProcess($name, $rows){
		switch ($name){
			case 'postlist':
				foreach ($rows->r as $r){
					if ($r->f == 'a'){ $this->PostAppend($r->d); }
					if ($r->f == 'u'){ $this->PostUpdate($r->d); }
				}
				break;
			case 'deptlist':
				foreach ($rows->r as $r){
					if ($r->f == 'a'){ $this->DeptAppend($r->d); }
					if ($r->f == 'u'){ $this->DeptUpdate($r->d); }
				}
				break;
			case 'employee':
				foreach ($rows->r as $r){
					if ($r->f == 'a'){ $this->EmployeeAppend($r->d); }
					if ($r->f == 'u'){ $this->EmployeeUpdate($r->d); }
				}
				break;
			case 'employeelist':
				foreach ($rows->r as $r){
					if ($r->f == 'd'){ $this->EmployeeRemove($r->d->id); }
				}
				break;
			case 'calperm':
				foreach ($rows->r as $r){
					if ($r->f == 'u'){ 
						$this->CalendarOptionsUpdate($rows->p->userid, $r->d); 
					}
				}
				break;
			case 'task':
				foreach ($rows->r as $r){
					if ($r->f == 'a'){ 
						$this->CalendarTaskAppend($r->d); 
					}
					if ($r->f == 'u'){ 
						$this->CalendarTaskUpdate($r->d); 
					}
					if ($r->f == 'd'){ 
						$this->CalendarTaskRemove($r->d->id); 
					}
				}
				break;
			case 'userconfig':
				foreach ($rows->r as $r){
					if ($r->f == 'u'){ 
						$this->UserConfigUpdate($r->d);
					}
					if ($r->f == 'a'){
						$this->UserConfigAppend($r->d);
					} 
				}
				break;
				
			default:
				$this->GetCalendarManager()->DSProcess($name, $rows);
				break;
		}
	}
	
	public function DSGetData($name, $rows){
		$p = $rows->p;
		switch ($name){
			case 'days':
				return $this->CalendarDays($p->uid, $p->days); 
			case 'employeelist':
				return $this->EmployeeList(); 
			case 'employee':
				return $this->Epmloyee($p->userid);
			case 'postlist':
				return $this->PostList(); 
			case 'calperm':
				return $this->CalendarPermission($p->userid); 
			case 'deptlist':
				return $this->DeptList(); 
			case 'userconfig':
				return $this->UserConfigList(); 
			default:
				return $this->GetCalendarManager()->DSGetData($name, $rows);
		}
		
		return null;
	}
	
	private function UserConfigCheckVarName($name){
		if (!$this->IsWriteRole()){ return false; }
		switch($name){
			case "se-myfilter":
				return true;
		}
		return false;
	}
	
	private function UserConfigCheckAccess($id){
		$info = UserQueryExt::UserConfigInfo($this->db, $id);
		
		if (empty($info) || $info['uid'] != $this->userid){ 
			return false; 
		}
		return true;
	}
	
	public function UserConfigList(){
		if (!$this->IsWriteRole()){ return null; }
		return UserQueryExt::UserConfigList($this->db, 'company', $this->userid);
	}

	public function UserConfigAppend($d){
		if (!$this->UserConfigCheckVarName($d->nm)){ return; }
		UserQueryExt::UserConfigAppend($this->db, 'company', $this->userid, $d->nm, $d->vl);
	}
	
	public function UserConfigUpdate($d){
		if (!$this->UserConfigCheckVarName($d->nm) || 
			!$this->UserConfigCheckAccess($d->id)){
			return;
		}
		UserQueryExt::UserConfigUpdate($this->db, $d->id, $d->nm, $d->vl);
	}
	
	public function UserConfigRemove($id){
		 if (!$this->UserConfigCheckAccess($id)){ return; }
	}
	
	public function CalendarTaskAppend($d){
		if (!$this->IsViewRole()){ return null; }
		$calMan = $this->GetCalendarManager();
		$d->own = 'cmpn';
		$calMan->TaskAppend($d);
	}
	
	public function CalendarTaskUpdate($d){
		if (!$this->IsViewRole()){ return null; }
		$calMan = $this->GetCalendarManager();
		$calMan->TaskUpdate($d);
	}
	
	public function CalendarTaskRemove($taskid){
		if (!$this->IsViewRole()){ return null; }
		$calMan = $this->GetCalendarManager();
		$calMan->TaskRemove($taskid);
	}
	
	public function CalendarDays($userid, $data){
		if (!$this->IsViewRole()){ return null; }
		$this->GetCalendarManager();
		
		if ($this->userid == $userid){
			return CalendarQuery::Days($this->db, $userid, $data);
		}
			
		$cemp = CompanyQuery::Employee($this->db, $this->userid, true);
		$emp = CompanyQuery::Employee($this->db, $userid, true);
		
		if (empty($cemp) || empty($emp) || $cemp['nop'] > 0 || $emp['nop'] > 0){
			return null;
		}
		if ($emp['lvl'] < $cemp['lvl']){
			if (!CompanyQuery::CalendarCheckAccess($this->db, $emp['id'], $cemp['id'])){
				return null;
			}
		} 
	
		return CompanyQuery::Days($this->db, $userid, $data);
	}
	
	public function CalendarOptionsUpdate($userid, $d){
		if (!$this->IsViewRole()){ return null; }
		if ($this->userid != $userid){ return null; }
		$empinfo = CompanyQuery::Employee($this->db, $userid, true);
		$curr = CompanyQuery::CalendarPermission($this->db, $userid, true);
		$calPermId = $curr['cpid'] * 1;
		
		// @TODO Необходимо организовать проверку входных данных
		if ($calPermId > 0){
			CompanyQuery::CalendarOptionsUpdate($this->db, $empinfo['id'], $d);
		}else{
			CompanyQuery::CalendarOptionsAppend($this->db, $empinfo['id'], $d);
		}
	}
	
	public function CalendarPermission($userid){
		if (!$this->IsViewRole()){ return null; }
		if ($this->userid != $userid){ return null; }
		return CompanyQuery::CalendarPermission($this->db, $userid);
	}
	
	public function EmployeeRemove($userid){
		if (!$this->IsManagerRole()){ return; }
		$userManager = Abricos::$user->GetManager();
		UserQueryExt::UserGroupRemoveByKey($this->db, $userid, CompanyGroup::EMPLOYEE);
	}
	
	public function EmployeeUpdate($d){
		if (!$this->IsManagerRole()){ return; }
		CompanyQuery::EmployeeUpdate($this->db, $d);
		$userManager = Abricos::$user->GetManager();
		UserQueryExt::UserGroupAppendByKey($this->db, $d->id, CompanyGroup::EMPLOYEE);
	}
	
	public function EmployeeAppend($d){
		$db = $this->db;
		$ret = array();
		$ret['err'] = 101;
		if (!$this->IsManagerRole()){ return $ret; }
		$username = $d->unm;
		
		if (empty($username)){ return $ret; }
		$userManager = Abricos::$user->GetManager();
		$userManager->DisableRoles();
		
		$user = UserQueryExt::UserByName($db, $username);
		if (empty($user)){
			// пользователь с такой учеткой не найден, регистрируем его
			$ret['err'] = $userManager->Register($username, $d->upwd, $d->ueml, false, false);
			if ($ret['err'] > 0){ return $ret; }
			$user = UserQueryExt::UserByName($db, $username);
		}else{
			$emp = CompanyQuery::Employee($db, $user['userid'], true);
			if (intval($emp['isemp']) > 0){
				$ret['err'] = 102;
				return $ret;
			}
		}
		$d->id = $user['userid'];
		CompanyQuery::EmployeeUpdate($db, $d);
		UserQueryExt::UserGroupAppendByKey($db, $d->id, CompanyGroup::EMPLOYEE);
		
		$ret['err'] = 0;
		return $ret;
	}
	
	public function Epmloyee($userid){
		if (!$this->IsViewRole()){ return null; }
		return CompanyQuery::Employee($this->db, $userid);
	}
	
	public function EmployeeList(){
		if (!$this->IsViewRole()){ return null; }
		return CompanyQuery::EmployeeList($this->db);
	}
	
	public function PostList(){
		if (!$this->IsViewRole()){ return null; }
		return CompanyQuery::PostList($this->db);
	}
	
	public function PostAppend($data){
		if (!$this->IsManagerRole()){ return null; }
		CompanyQuery::PostAppend($this->db, $data);
	}

	public function PostUpdate($data){
		if (!$this->IsManagerRole()){ return null; }
		CompanyQuery::PostUpdate($this->db, $data);
	}

	public function DeptList(){
		if (!$this->IsViewRole()){ return null; }
		return CompanyQuery::DeptList($this->db);
	}
	
	public function DeptAppend($data){
		if (!$this->IsManagerRole()){ return null; }
		CompanyQuery::DeptAppend($this->db, $data);
	}

	public function DeptUpdate($data){
		if (!$this->IsManagerRole()){ return null; }
		CompanyQuery::DeptUpdate($this->db, $data);
	}
	
}


?>