<?php 
/**
 * Модуль "Компания"
 * 
 * @version $Id: module.php 1413 2012-02-02 09:09:11Z roosit $
 * @package Abricos 
 * @subpackage Company
 * @copyright Copyright (C) 2008 Abricos All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin (roosit@abricos.org)
 */

class CompanyModule extends Ab_Module {
	
	private $_companyManager = null;
	
	public function __construct(){
		// Версия модуля
		$this->version = "0.1.5";
		
		// Название модуля
		$this->name = "company";
		
		$this->permission = new CompanyPermission($this);
	}
	
	public function GetCompanyManager(){
		if (is_null($this->_companyManager)){
			require_once 'includes/manager.php';
			$this->_companyManager = new CompanyManager($this);
		}
		return $this->_companyManager;
	}
}

class CompanyAction {
	/**
	 * Действия: просмотр списка сотрудников
	 * @var integer
	 */
	const VIEW = 10;
	
	/**
	 * Действия: все, что разрешено сотруднику в рамках модуля компании
	 * @var integer
	 */
	const WRITE = 30;
	
	/**
	 * Действия: соответствует действиям отдела кадров (добавление, редактирование, удаление сотрудников)
	 * @var integer
	 */
	const MANAGER = 40;
	
	/**
	 * Действия: администрирование модуля 
	 * @var integer
	 */
	const ADMIN = 50;
}

class CompanyGroup {
	
	/**
	 * Группа "Сотрудники"
	 * @var string
	 */
	const EMPLOYEE = 'company_employee';
	
}

class CompanyPermission extends Ab_UserPermission {
	
	public function __construct(CompanyModule $module){
		
		$defRoles = array(
			new Ab_UserRole(CompanyAction::VIEW, Ab_UserGroup::GUEST),
			new Ab_UserRole(CompanyAction::VIEW, Ab_UserGroup::REGISTERED),
			new Ab_UserRole(CompanyAction::VIEW, Ab_UserGroup::ADMIN),
			
			new Ab_UserRole(CompanyAction::WRITE, CompanyGroup::EMPLOYEE),
			
			new Ab_UserRole(CompanyAction::MANAGER, Ab_UserGroup::ADMIN),
			
			new Ab_UserRole(CompanyAction::ADMIN, Ab_UserGroup::ADMIN)
		);
		
		parent::__construct($module, $defRoles);
	}
	
	public function GetRoles(){
		return array(
			CompanyAction::VIEW => $this->CheckAction(CompanyAction::VIEW),
			CompanyAction::WRITE => $this->CheckAction(CompanyAction::WRITE),
			CompanyAction::MANAGER => $this->CheckAction(CompanyAction::MANAGER),
			CompanyAction::ADMIN => $this->CheckAction(CompanyAction::ADMIN) 
		);
	}
}

Abricos::ModuleRegister(new CompanyModule());

?>