<?php
/**
 * @version $Id: manager.php 698 2010-09-06 12:37:21Z roosit $
 * @package Abricos
 * @subpackage EPrice
 * @copyright Copyright (C) 2008 Abricos. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin (roosit@abricos.org)
 */

require_once 'dbquery.php';

class EPriceManager extends Ab_ModuleManager {
	
	/**
	 * 
	 * @var EPriceModule
	 */
	public $module = null;
	
	private $_disableRoles = false;
	
	public function __construct(EPriceModule $module){
		parent::__construct($module);
	}
	
	public function DisableRole(){
		$this->_disableRoles = true; 
	}
	
	/**
	 * Получить менеджер каталога
	 * 
	 * @return CatalogManager
	 */
	public function GetCatalogManager(){
		return Abricos::GetModule('eshop')->GetCatalogManager();
	}
	
	public function IsAdminRole(){
		return $this->IsRoleEnable(EPriceAction::ADMIN);
	}
	
	public function IsWriteRole(){
		return $this->IsRoleEnable(EPriceAction::WRITE);
	}
	
	public function IsViewRole(){
		return $this->IsRoleEnable(EPriceAction::VIEW);
	}
	
	public function DSProcess($name, $rows){
		$p = $rows->p;
		$db = $this->db;
		
		switch ($name){
			case 'productlist':
				foreach ($rows->r as $r){
					if ($r->f == 'u'){ $this->ProductUpdate($r->d); }
				}
				return;
		}
	}
	
	public function DSGetData($name, $rows){
		$p = $rows->p;
		switch ($name){
			case 'productlist': return $this->ProductList();
			case 'eloption': return $this->ElementOptionList();
		}
	}
	
	public function AJAX($d){
		switch($d->do){
			// case "orderstomake": return $this->OrdersToMake($d->makedate);
		}
		return null;
	}
	
	private $_priceFieldList = null;
	private function PriceFieldList(){
		if (!is_null($this->_priceFieldList)){
			return $this->_priceFieldList;
		}
		$catman = $this->GetCatalogManager();
		$fields = $catman->ElementOptionListArray();
		$ret = array();
		foreach($fields as $id => $row){
			if ($row['fldtp'] != CatalogOptionType::DOUBLE){ continue; }
			$row['servnm'] = "fld_".$row['nm'];
			$ret[$id] = $row;
		}
		$this->_priceFieldList = $ret;
		return $ret;
	}
	
	public function ElementOptionList(){
		$catman = $this->GetCatalogManager();
		return $catman->ElementOptionList();
	}
	
	private function CatalogElementList(){
		$catman = $this->GetCatalogManager();
		
		$sel = array(
			"fld_sklad as cnt",
			"fld_ord as ord"
		);
		$fields = $this->PriceFieldList();
		foreach($fields as $id => $row){
			array_push($sel, $row['servnm']." as ".$row['nm']);
		}
		
		return $catman->ElementList(0, 1, 1000, '1>0', "fld_ord, fld_name",
			implode(", ", $sel) 
			// "fld_price as pc, fld_price_opt as pco, fld_sklad as cnt"
		);
	}
	
	public function ProductList(){
		if (!$this->IsViewRole()){ return; }
		return $this->CatalogElementList();
	}
	
	private $_productList = null;
	public function ProductListArray(){
		if (!is_null($this->_productList)){ return $this->_productList; }
		$rows = $this->CatalogElementList();
		while (($row = $this->db->fetch_array($rows))){
			$this->_productList[$row['id']] = $row;
		}
		return $this->_productList;
	}
	
	public function ProductUpdate($d){
		if (!$this->IsWriteRole()){ return; }
		$catman = $this->GetCatalogManager();
		$data = new stdClass();
		$data->id = $d->id;
		$data->elid = $d->elid;
		$data->catid = $d->catid;
		$data->eltid = $d->eltid;
		$data->tl = $d->tl;
		$data->nm = $d->nm;
		$data->fld_sklad = $d->cnt;
		$data->fld_ord = $d->ord;
		
		$fields = $this->PriceFieldList();
		foreach($fields as $id => $row){
			$data->$row['servnm'] = $d->$row['nm'];
		}
		
		$catman->ElementUpdate($data, false);
	}
	
	
	
}

?>