/*
@version $Id: project.js 694 2010-08-26 07:31:16Z roosit $
@package Abricos
@copyright Copyright (C) 2008 Abricos All rights reserved.
@license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

var Component = new Brick.Component();
Component.requires = {
	yahoo: ['dom'],
	mod:[
		{name: 'sys', files: ['data.js', 'container.js']},
        {name: 'eprice', files: ['lib.js']}
	]
};
Component.entryPoint = function(){
	
	var Dom = YAHOO.util.Dom,
		E = YAHOO.util.Event,
		L = YAHOO.lang,
		NS = this.namespace, 
		TMG = this.template,
		API = NS.API;

	if (!NS.data){
		NS.data = new Brick.util.data.byid.DataSet('eprice');
	}
	
	var buildTemplate = function(w, templates){
		var TM = TMG.build(templates), T = TM.data, TId = TM.idManager;
		w._TM = TM; w._T = T; w._TId = TId;
	};
	
	var ProductList = function(){
		this.init();
	};
	ProductList.prototype = {
		init: function(){
			
			var D = NS.data;
			D.onStart.subscribe(this.dsEvent, this, true);
			D.onComplete.subscribe(this.dsEvent, this, true);
			
			this.onRender = new YAHOO.util.CustomEvent("onRender");
			this.onRenderWait = new YAHOO.util.CustomEvent("onRenderWait");
			
			D.isFill({
				'productlist': D.get('productlist', true)
			}) ? this.render() : this.renderWait();
		},
		dsEvent: function(type, args){
			if (args[0].checkWithParam('productlist', {})){
				type == 'onComplete' ? this.render() : this.renderWait(); 
			}
		},
		destroy: function(){
			NS.data.onComplete.unsubscribe(this.dsEvent);
			NS.data.onStart.unsubscribe(this.dsEvent);
		},
		render: function(){
			this.onRender.fire();
		},
		renderWait: function(){
			this.onRenderWait.fire();
		},
		foreach: function(func){
			if (!L.isFunction(func)){ return; }
			NS.data.get('productlist').getRows().foreach(function(row){
				func(row);
			});
		}
	};
	NS.ProductList = ProductList;


	var ProductSelectWidget = function(container){
		this.init(container);
	};
	ProductSelectWidget.prototype = {
		init: function(container){
			this.container = Dom.get(container);
			this.selectedValue = null;
			this.filter = null;
			
			this.onChange = new YAHOO.util.CustomEvent("onChange");
			
			buildTemplate(this, 'compseltable,compselrow,compselrowwait');
			
			this.tables = new Brick.mod.sys.TablesManager(NS.data, ['productlist'], {'owner': this});
		},
		destroy: function(){
			this.tables.destroy();
		},
		onDataLoadWait: function(tables){ 
			var TM = this._TM, T = this._T;
			this.container.innerHTML = TM.replace('compseltable', {'rows': T['compselrowwait']});
		},
		onDataLoadComplete: function(tables){
			this.render();
		},
		render: function(){
			var lst = "", TM = this._TM, T = this._T;
			
			var filter = L.isFunction(this.filter) ? this.filter : function(){ return true; };

			NS.data.get('productlist').getRows().foreach(function(row){
				if (!filter(row)){ return; }
				var di = row.cell;
				lst += TM.replace('compselrow', {
					'id': row.id,
					'tl': di['tl']
				});
			});
			this.container.innerHTML = TM.replace('compseltable', {'rows': lst});
			
			if (L.isNull(this.selectedValue)){
				var row = NS.data.get('productlist').getRows().getByIndex(0);
				if (!L.isNull(row)){
					this.setValue(row.cell['id']);
				}
			}else{
				this.setValue(this.selectedValue);
			}
			
			var el = TM.getEl('compseltable.id'), __self = this;
			E.on(el, 'change', function(){ __self._onChange(el.value); });
		},
		setFilter: function(f){
			this.filter = f;
			this.render();
		},
		_onChange: function(value){
			this.selectedValue = value;
			this.onChange.fire(value);
		},
		getValue: function(){
			return this._TM.getEl('compseltable.id').value;
		},
		setValue: function(value){
			value = value*1;
			this.selectedValue = value;
			this._TM.getEl('compseltable.id').value = value;
		}
	};
	NS.ProductSelectWidget = ProductSelectWidget;

};