<?php
/**
 * @version $Id: manager.php 1413 2012-02-02 09:09:11Z roosit $
 * @package Abricos
 * @subpackage Feedback2
 * @copyright Copyright (C) 2008 Abricos. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Gabidullin Mansur (mansur.gabidullin@gmail.com)
 */

require_once 'dbquery.php';

class Feedback2Manager extends Ab_ModuleManager {
	
	/**
	 * 
	 * @var Feedback2Module
	 */
	public $module = null;
	
	public $username = "";
	public $useremail = "";
	
	public function __construct(Feedback2Module $module){
		parent::__construct($module);
		
		$this->username = Abricos::$user->info['username'];
		$this->useremail = Abricos::$user->info['email'];
	}
	
	public function IsAdminRole(){
		return $this->IsRoleEnable(Feedback2Action::FB2_ADMIN);
	}
	
	public function IsRemoveRole(){
		return $this->IsRoleEnable(Feedback2Action::FB2_REMOVE);
	}
	
	public function IsWriteRole(){
		return $this->IsRoleEnable(Feedback2Action::FB2_WRITE);
	}
	
	public function IsModeratorRole(){
		return $this->IsRoleEnable(Feedback2Action::FB2_MODERATOR);
	}	
	
	public function IsStatusChangeRole(){
		return $this->IsRoleEnable(Feedback2Action::FB2_STATUS_CHANGE);
	}
	
	public function IsViewRole(){
		return $this->IsRoleEnable(Feedback2Action::FB2_VIEW);
	}
	
	private $_lastAppendFeedbackId = 0;
	public function DSProcess($name, $rows){
		$p = $rows->p;
		$db = $this->db;
		
		switch ($name){
			case 'all':
			case 'ideas':
			case 'questions':
			case 'errors':
			case 'thanks':
				foreach ($rows->r as $r){
					if ($r->f == 'a'){
						$this->_lastAppendFeedbackId = $this->FeedbackAppend($r->d);
					}
					if ($r->f == 'u'){ $this->FeedbackUpdate($r->d); }
					if ($r->f == 'd'){ $this->FeedbackRemove($r->d->id); }
				}
				return;
			case 'statuslist':
				foreach ($rows->r as $r){
					if ($r->f == 'a'){ $this->StatusAppend($r->d); }
					if ($r->f == 'u'){ $this->StatusUpdate($r->d); }
					if ($r->f == 'd'){ $this->StatusRemove($r->d->id); }
				}
				return;
		}
	}
	
	public function DSGetData($name, $rows){
		$p = $rows->p;
		switch ($name){			
			case 'ui-top': return $this->TopTable(10);
			case 'ui-all': return $this->AllTable();
			case 'ui-ideas': return $this->IQETTable('idea');
			case 'ui-questions': return $this->IQETTable('questions');
			case 'ui-errors': return $this->IQETTable('errors');
			case 'ui-thanks': return $this->IQETTable('thanks');
			case 'toptable': return $this->TopTable($p->limit);
			case 'all': return $this->FeedbackList($p->page, $p->limit, $p->sort, -1);
			case 'ideas': return $this->FeedbackList($p->page, $p->limit, $p->sort, 0);
			case 'questions': return $this->FeedbackList($p->page, $p->limit, $p->sort, 1);
			case 'errors': return $this->FeedbackList($p->page, $p->limit, $p->sort, 2);
			case 'thanks': return $this->FeedbackList($p->page, $p->limit, $p->sort, 3);
			case 'all-count': return $this->FeedbackCount(-1);
			case 'ideas-count': return $this->FeedbackCount(0);
			case 'questions-count': return $this->FeedbackCount(1);
			case 'errors-count': return $this->FeedbackCount(2);
			case 'thanks-count': return $this->FeedbackCount(3);
		}
		return null;
	}
	
	public function AJAX($d){
		switch($d->do){
			case "add":			return $this->TaskAppend($d);
			case "up":			return $this->VoteUp($d);
			case "down":		return $this->VoteDown($d);
			case "checkvote":	return $this->VoteCheck($d);
			case 'gettip':		return $this->GetTip($d);
			case 'getmodertip':	return $this->GetModerTip($d);
			case 'moderchecked':	return $this->FeedbackUpdate($d->d);
		}
		return -1;
	}
	
	public function TaskAppend($d){
		if (!$this->IsViewRole()){ return -1; }
		$utmanager = Abricos::TextParser();
		$title = $utmanager->Parser($d->title);
		$body = $utmanager->Parser($d->body);
		$email = $utmanager->Parser($d->email);
		
		if($body=='' || $title=='')return -1;
		$task = new Feedback2Task();
		$task->userid = $id;
		$task->username = $this->username;
		
		if($task->userid>0){
			$task->useremail = $email != '' ? $email : 'Не указан';
		}else $task->useremail = $this->useremail;
		
		$task->title = $title;
		$task->body = $body;
		$task->sectionid = bkint($d->sectionid);
		$insertid = Feedback2Query::TaskAppend($this->db, $task);
		$this->FeedbackSendNotify($insertid);
		return $insertid;
	}
	
	public function VoteUp($d){
		if (!$this->IsViewRole()){ return -1; }
		return Feedback2Query::VoteUp($this->db, $this->userid, $d); 
	}
	
	public function VoteDown($d){
		if (!$this->IsViewRole()){ return -1; }
		return Feedback2Query::VoteDown($this->db, $this->userid, $d); 
	}
	
	public function VoteCheck($d){
		if (!$this->IsViewRole()){ return -1; }
		return Feedback2Query::VoteCheck($this->db, $this->userid, $d); 
	}
	
	public function FeedbackAppend($d){
		if (!$this->IsAdminRole() && !$this->IsModeratorRole()){ return; }
		return Feedback2Query::FeedbackAppend($this->db, $this->userid, $this->username, $this->useremail, $d);
	}
	
	public function FeedbackUpdate($d){
		if (!$this->IsAdminRole() && !$this->IsModeratorRole()){ return; }
		return Feedback2Query::FeedbackUpdate($this->db, $this->userid, $this->username, $d);
	}
	
	public function FeedbackRemove($ideaid){
		if (!$this->IsAdminRole() && !$this->IsModeratorRole()){ return; }
		return Feedback2Query::FeedbackRemove($this->db, $ideaid);
	}
	
	public function TopTable($limit){
		if (!$this->IsViewRole()){ return -1; }
		return Feedback2Query::TopTable($this->db, $limit);
	}
	
	public function IQETTable($iqet){
		if (!$this->IsViewRole()){ return -1; }
		return Feedback2Query::IQETTable($this->db, $iqet);
	}
	
	public function AllTable(){
		if (!$this->IsViewRole()){ return -1; }
		return Feedback2Query::AllTable($this->db);
	}
		
	public function FeedbackList($page, $limit, $sort, $mode){
		if (!$this->IsModeratorRole()){ return -1; }
		return Feedback2Query::FeedbackList($this->db, $page, $limit, $sort, $mode);
	}
	
	public function FeedbackCount($mode){
		if (!$this->IsModeratorRole()){ return -1; }
		return Feedback2Query::FeedbackCount($this->db, $mode);
	}
	
	public function GetTip($d){
		if (!$this->IsViewRole()){ return -1; }
		return Feedback2Query::GetTip($this->db, $d, $this->userid, true);
	}
	
	public function GetModerTip($d){
		if (!$this->IsModeratorRole()){ return -1; }
		return Feedback2Query::GetModerTip($this->db, $d, $this->userid, true);
	}
	
	//комментарии
	public function IsCommentList($contentid){
		return true;
	}
	
	public function IsCommentAppend($contentid){
		return true;
	}
	
	private function ToArray($rows){
		$ret = array();
		while (($row = $this->db->fetch_array($rows))){
			$ret[$row['id']] = $row;
		}
		return $ret;
	}
	
	/**
	 * Отправить модератору уведомление о новом отзыве.
	 * 
	 * @param int $fbid
	 */
	public function FeedbackSendNotify($fbid){
		//	$fbid - идентификатор комментария
		
		$brick = Brick::$builder->LoadBrickS('feedback2', 'templates', null, null);
		$host = $_SERVER['HTTP_HOST'] ? $_SERVER['HTTP_HOST'] : $_ENV['HTTP_HOST'];
		$plnk = "http://".$host."/webos/start/feedback2/manager/showFeedbackEditorPanel/".$fbid."/";
		
		$emails = array();
		$users = $this->ToArray(Feedback2Query::GetModers($this->db));
		foreach ($users as $user){
			$email = $user['eml'];
			
			if (empty($email) || $emails[$email] || $user['id'] == $this->userid){
				continue;
			}
			$emails[$email] = true;
			$subject = Brick::ReplaceVarByData($brick->param->var['fbemlnewsubject'], array(
				"unm" => $this->user->info['username']
			));
			$body = Brick::ReplaceVarByData($brick->param->var['fbemlnewbody'], array(
				"tl" => Feedback2Query::GetFeedbackTitle($this->db, $fbid),
				"plnk" => $plnk,
				"sitename" => Brick::$builder->phrase->Get('sys', 'site_name')
			));
			Abricos::Notify()->SendMail($email, $subject, $body);
		}
		
		
		return true;
		
		// уведомление подписчикам
		$users = $this->ToArray(BoprosQuery::ProjectSubscribeUserList($this->db, $prj['id']));
		foreach ($users as $user){
			$email = $user['email'];
			
			if (empty($email) || $emails[$email] || $user['id'] == $this->userid){
				continue;
			}
			$emails[$email] = true;
			$subject = Brick::ReplaceVarByData($brick->param->var['fbemlnewsubject'], array(
				"unm" => $this->user->info['username']
			));
			$body = Brick::ReplaceVarByData($brick->param->var['fbemlnewbody'], array(
				"tl" => $cht['tl'],
				"plnk" => $plnk,
				"sitename" => Brick::$builder->phrase->Get('sys', 'site_name')
			));
			Abricos::Notify()->SendMail($email, $subject, $body);
		}
		
		
		
		
			$subject = Brick::ReplaceVarByData($brick->param->var['cmtemlsubject'], array(
				"tl" => $prj['tl']
			));
			$body = Brick::ReplaceVarByData($brick->param->var['cmtemlbody'], array(
				"tl" => $prj['tl'],
				"plnk" => $plnk,
				"unm" => $this->user->info['username'],
				"cmt" => $data->bd." ",
				"sitename" => Brick::$builder->phrase->Get('sys', 'site_name')
			));
			Abricos::Notify()->SendMail($email, $subject, $body);
	}
	
	/**
	 * Отправить уведомление о новом комментарии.
	 * 
	 * @param object $data
	 */
	public function CommentSendNotify($data){
		// данные по комментарию:
		// $data->id	- идентификатор комментария
		// $data->pid	- идентификатор родительского комментария
		// $data->uid	- пользователь оставивший комментарий
		// $data->bd	- текст комментария
		// $data->cid	- идентификатор контента
		
		$cht = Feedback2Query::ChitByContentId($this->db, $data->cid, true);
		if (empty($cht)){ return; }
		
		$emails = array();
		
		$brick = Brick::$builder->LoadBrickS('feedback2', 'templates', null, null);
		$host = $_SERVER['HTTP_HOST'] ? $_SERVER['HTTP_HOST'] : $_ENV['HTTP_HOST'];
		$plnk = "http://".$host."/webos/start/feedback2/ui/showViewPanel/".$cht['id']."/";
		
		// уведомление "комментарий на комментарий"
		if ($data->pid > 0){
			$parent = CommentQuery::Comment($this->db, $data->pid, $data->cid, true);
			if (!empty($parent) && $parent['uid'] != $this->userid){
				$user = UserQuery::User($this->db, $parent['uid']);
				$email = $user['email'];
				if (!empty($email)){
					$emails[$email] = true;
					$subject = Brick::ReplaceVarByData($brick->param->var['cmtemlsubject'], array(
						"tl" => $cht['tl']
					));
					$body = Brick::ReplaceVarByData($brick->param->var['cmtemlbody'], array(
						"tl" => $cht['tl'],
						"plnk" => $plnk,
						"unm" => $this->user->info['username'],
						"cmt1" => $parent['bd']." ",
						"cmt2" => $data->bd." ",
						"sitename" => Brick::$builder->phrase->Get('sys', 'site_name')
					));
					Abricos::Notify()->SendMail($email, $subject, $body);
				}
			}
		}
		
		// уведомление автору
		if ($cht['uid'] != $this->userid){
			$autor = UserQuery::User($this->db, $cht['uid']);
			$email = $autor['email'];
			if (!empty($email) && !$emails[$email]){
				$emails[$email] = true;
				$subject = Brick::ReplaceVarByData($brick->param->var['cmtemlautorsubject'], array(
					"tl" => $cht['tl']
				));
				$body = Brick::ReplaceVarByData($brick->param->var['cmtemlautorbody'], array(
					"tl" => $cht['tl'],
					"plnk" => $plnk,
					"unm" => $this->user->info['username'],
					"cmt" => $data->bd." ",
					"sitename" => Brick::$builder->phrase->Get('sys', 'site_name')
				));
				Abricos::Notify()->SendMail($email, $subject, $body);
			}
		}
	}
}

class Feedback2Task {
	public $userid = 0;
	public $username = '';
	public $useremail = '';
	public $statusid = 0;
	public $title = '';
	public $body = '';
	public $reply = '';
	public $contentid = 0;
	public $sectionid = 1;
}

?>