<?php
/**
 * Модуль "Отзывы пользователей"
 *
 * @version $Id: module.php 1413 2012-02-02 09:09:11Z roosit $
 * @package Abricos
 * @subpackage Feedback2
 * @copyright Copyright (C) 2008 Abricos All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Gabidullin Mansur (mansur.gabidullin@gmail.com)
 */
class Feedback2Module extends Ab_Module {
	
	function __construct(){
		$this->version = "0.1.2";
		$this->name = "feedback2";
		
		$this->permission = new Feedback2Permission($this);
	}
	
	private $_manager;
	
	/**
	 * Получить менеджер
	 *
	 * @return Feedback2Manager
	 */
	public function GetManager(){
		if (is_null($this->_manager)){
			require_once 'includes/manager.php';
			$this->_manager = new Feedback2Manager($this);
		}
		return $this->_manager;
	}
}

class Feedback2Action {
	const FB2_VIEW			= 10;
	const FB2_WRITE			= 20;
	const FB2_REMOVE		= 30;
	const FB2_STATUS_CHANGE	= 40;
	const FB2_ADMIN			= 50;
	const FB2_MODERATOR		= 60;
}

class Feedback2Group {
	
	/**
	 * Группа "Модератор отзывов пользователей"
	 * @var string
	 */
	const MODERATOR = 'feedback2_moderator';
	
}

class Feedback2Permission extends Ab_UserPermission {
	
	public function Feedback2Permission(Feedback2Module $module){
		
		$defRoles = array(
			new Ab_UserRole(Feedback2Action::FB2_VIEW, Ab_UserGroup::GUEST),
			new Ab_UserRole(Feedback2Action::FB2_VIEW, Ab_UserGroup::REGISTERED),
			new Ab_UserRole(Feedback2Action::FB2_VIEW, Ab_UserGroup::ADMIN),
			
			new Ab_UserRole(Feedback2Action::FB2_WRITE, Ab_UserGroup::GUEST),
			new Ab_UserRole(Feedback2Action::FB2_WRITE, Ab_UserGroup::REGISTERED),
			new Ab_UserRole(Feedback2Action::FB2_WRITE, Ab_UserGroup::ADMIN),
			
			new Ab_UserRole(Feedback2Action::FB2_STATUS_CHANGE, Ab_UserGroup::ADMIN),			
			new Ab_UserRole(Feedback2Action::FB2_ADMIN, Ab_UserGroup::ADMIN),
			new Ab_UserRole(Feedback2Action::FB2_MODERATOR, Ab_UserGroup::ADMIN),
			new Ab_UserRole(Feedback2Action::FB2_MODERATOR, Feedback2Group::MODERATOR)
		);
		parent::__construct($module, $defRoles);
	}
	
	public function GetRoles(){
		return array(
			Feedback2Action::FB2_VIEW => $this->CheckAction(Feedback2Action::FB2_VIEW),
			Feedback2Action::FB2_WRITE => $this->CheckAction(Feedback2Action::FB2_WRITE),
			Feedback2Action::FB2_STATUS_CHANGE => $this->CheckAction(Feedback2Action::FB2_STATUS_CHANGE), 
			Feedback2Action::FB2_ADMIN => $this->CheckAction(Feedback2Action::FB2_ADMIN),
			Feedback2Action::FB2_MODERATOR => $this->CheckAction(Feedback2Action::FB2_MODERATOR)
		);
	}
}

Abricos::ModuleRegister(new Feedback2Module());

?>